/* goto-dialog.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Valide.GotoDialog : Gtk.Window
{
  private Gtk.Entry entry;

  public AbstractTextDocument document
  {
    construct;
    get;
  }

  private void hide_search (bool cancel)
  {
    this.hide ();

    if (cancel)
    {
    }
  }

  private void update_position ()
  {
	  int view_x, view_y;
	  Gdk.Window view_window = this.document.text_view.window;

	  this.realize ();

	  view_window.get_origin (out view_x, out view_y);
  
	  this.move (view_x, view_y);
  }

  public GotoDialog (AbstractTextDocument document)
  {
    this.document = document;
    this.type = Gtk.WindowType.POPUP;
  }

  construct
  {
    this.set_modal (true);

    Gtk.Frame frame = new Gtk.Frame (null);
    frame.set_shadow_type (Gtk.ShadowType.ETCHED_IN);
    this.add (frame);
    frame.show ();

    Gtk.VBox vbox = new Gtk.VBox (false, 0);
    vbox.set_border_width (3);
    frame.add (vbox);
    vbox.show ();

    this.entry = new Gtk.Entry ();
    vbox.add (entry);
    this.entry.show ();
    this.entry.realize ();

    this.entry.activate += () => {
      this.hide_search (false);
    };

    this.entry.insert_text += (s, t, l, p) => {
      int i = 0;
		  unichar c;

		  while (i < l) 
		  {
			  c = t[i];

			  if (!c.isdigit ())
			  {
				  Signal.stop_emission_by_name (s, "insert_text");
				  break;
			  }
			  i++;
		  }
    };

    this.entry.changed += () => {
      if (this.entry.text != "")
      {
        bool moved;
        int line;

        line = this.entry.text.to_int ();
        if (line > 0)
        {
           line--;
        }
        moved = this.document.goto_line (line);

        Gdk.Color color;

        if (!moved)
        {
      		Gdk.Color.parse ("#FFBFBF", out color);
        }
        else
        {
      		Gdk.Color.parse ("white", out color);
        }
        this.entry.modify_base (Gtk.StateType.NORMAL, color);
      }
    };

    this.delete_event += () => {
      this.hide_search (false);
      return false;
    };

    this.focus_out_event += () => {
      this.hide_search (true);
      return false;
    };

    this.key_press_event += (s, e) => {
      switch (e.keyval)
      {
        case Gdk.KeySyms.Tab:
          this.hide_search (false);
          return true;
        break;
        case Gdk.KeySyms.Escape:
          /* Cancel */
          this.hide_search (true);
          return true;
        break;
        default:
          return false;
        break;
      }
    };
  }

  public new void show ()
  {
    base.show ();
    this.update_position ();
    this.entry.grab_focus ();
  }
}

