/* plugin-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Valide.PluginManager : Object
{
  private static PluginManager instance;
  public List<PluginRegistrar> plugins;

  private void register_plugin (string filename) throws Error
  {
    PluginRegistrar registrar = new PluginRegistrar (filename);

    if (ConfigManager.get_instance ().get_boolean ("Plugins", registrar.name))
    {
      registrar.load();
    }
    this.plugins.append (registrar);
  }

  private PluginManager ()
  {
  }

  ~PluginManager()
  {
    foreach (PluginRegistrar pr in this.plugins)
    {
      pr.unload ();
    }
  }

  construct
  {
    string[] plugin_dir = { Config.PACKAGE_PLUGIN_DIR,
                            Path.build_filename (ConfigManager.get_instance ().home,
                                                 "plugins")
                          };

    foreach (string d in plugin_dir)
    {
      try
      {
        Dir dir = Dir.open (d);

        do
        {
          string filename = dir.read_name ();
          if (filename != null)
          {
            string file = Path.build_filename (d, filename);

            if (FileUtils.test (file, FileTest.IS_DIR))
            {
              file = Path.build_filename (file, filename + "." + PluginRegistrar.FILE_EXT[0]);
            }

            if (file.has_suffix ("." + PluginRegistrar.FILE_EXT[0]))
            {
              try
              {
                this.register_plugin (file);
              }
              catch (Error e)
              {
                warning (e.message);
              }
            }
          }
          else
          {
            break;
          }
        } while (true);
      }
      catch (Error e)
      {
        debug (e.message);
      }
    }
  }

  public static PluginManager get_instance ()
  {
    if (instance == null)
    {
      instance = new PluginManager ();
    }
    return instance;
  }

  public Gtk.Widget preferences ()
  {
    return new PluginPreferences (this);
  }

  public void free ()
  {
    this.instance = null;
  }
}

