/* project-dialog-options.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Valide.ProjectDialogOptions : AbstractProjectDialogOptions
{
  private Gtk.ListStore list_store_src;
  private Gtk.ListStore list_store_dst;
  private Gtk.ListStore list_store_vapi;

  /* Frame general */
  public string path
  {
    get { return this.widgets.path.get_text (); }
    set { this.widgets.path.set_text (value); }
  }

  public string name
  {
    get { return this.widgets.name.get_text (); }
    set { this.widgets.name.set_text (value); }
  }

  public string author
  {
    get { return this.widgets.author.get_text (); }
    set { this.widgets.author.set_text (value); }
  }

  public string license
  {
    get { return this.widgets.license.get_text (); }
    set { this.widgets.license.set_text (value); }
  }

  public string version
  {
    get { return this.widgets.version.get_text (); }
    set { this.widgets.version.set_text (value); }
  }

  /* Frame compiler */
  public bool debug
  {
    get { return this.widgets.debug.get_active (); }
    set { this.widgets.debug.set_active (value); }
  }

  public bool thread
  {
    get { return this.widgets.thread.get_active (); }
    set { this.widgets.thread.set_active (value); }
  }

  public bool disable_assert
  {
    get { return this.widgets.disable_assert.get_active (); }
    set { this.widgets.disable_assert.set_active (value); }
  }

  public bool disable_checking
  {
    get { return this.widgets.disable_checking.get_active (); }
    set { this.widgets.disable_checking.set_active (value); }
  }

  public bool disable_non_null
  {
    get { return this.widgets.disable_non_null.get_active (); }
    set { this.widgets.disable_non_null.set_active (value); }
  }

  public bool save_temps
  {
    get { return this.widgets.save_temps.get_active (); }
    set { this.widgets.save_temps.set_active (value); }
  }

  public bool quiet
  {
    get { return this.widgets.quiet.get_active (); }
    set { this.widgets.quiet.set_active (value); }
  }

  public string other
  {
    owned get { return this.widgets.other.get_text (); }
    set { this.widgets.other.set_text (value); }
  }

  /* Frame vapi dir */
  public List<string> vapi_dir
  {
    owned get
    {
      List<string> it;
      Gtk.TreeIter iter;
      string str;

      if (this.list_store_vapi.get_iter_first (out iter))
      {
        do
        {
          this.list_store_vapi.get (iter, 0, out str);
          it.append (str);
        } while (this.list_store_vapi.iter_next (ref iter));
      }
      return it;
    }

    set
    {
      foreach (string it in value)
      {
        Gtk.TreeIter iter;

        this.list_store_vapi.append (out iter);
        this.list_store_vapi.set (iter, 0, it);
      }
    }
  }

  /* @todo Check duplicated items */
  [CCode (instance_pos = -1)]
  protected void add_vapi_dir (Gtk.Widget sender)
  {
    string dirname = this.widgets.file_chooser_button.get_filename ();

    if (dirname != "")
    {
      Gtk.TreeIter iter;

      this.list_store_vapi.append (out iter);
      this.list_store_vapi.set (iter, 0, dirname);
      this.update_vapi ();
    }
  }

  [CCode (instance_pos = -1)]
  protected void del_vapi_dir (Gtk.Button sender)
  {
    Gtk.TreeIter iter;
    Gtk.TreeSelection selection = this.widgets.tree_view_vapi.get_selection ();

    if (selection.get_selected (null, out iter))
    {
      this.list_store_vapi.remove (iter);
      this.update_vapi ();
    }
  }


  /* Frame package */
  public List<string> pkg
  {
    owned get
    {
      List<string> it;
      Gtk.TreeIter iter;
      string str;

      if (this.list_store_dst.get_iter_first (out iter))
      {
        do
        {
          this.list_store_dst.get (iter, 0, out str);
          it.append (str);
        } while (this.list_store_dst.iter_next (ref iter));
      }
      return it;
    }

    set
    {
      foreach (string it in value)
      {
        Gtk.TreeIter iter;

        this.list_store_dst.append (out iter);
        this.list_store_dst.set (iter, 0, it);
      }
    }
  }

  private bool pkg_exist (string pkg)
  {
    bool exist = false;

    foreach (string p in this.pkg)
    {
      if (p == pkg)
      {
        exist = true;
        break;
      }
    }
    return exist;
  }

  public void update_vapi ()
  {
    List<string> pkg = Vala.get_package_list (this.vapi_dir);

    this.list_store_src.clear ();
    foreach (string p in pkg)
    {
      Gtk.TreeIter iter;

      this.list_store_src.append (out iter);
      this.list_store_src.set (iter, 0, p);
    }
  }

  [CCode (instance_pos = -1)]
  protected void add_pkg (Gtk.Button sender)
  {
    string pkg;
    Gtk.TreeIter iter;
    Gtk.TreeSelection selection = this.widgets.tree_view_src.get_selection ();

    if (selection.get_selected (null, out iter))
    {
      this.list_store_src.get (iter, 0, out pkg);
      if (!this.pkg_exist (pkg))
      {
        this.list_store_dst.append (out iter);
        this.list_store_dst.set (iter, 0, pkg);
      }
    }
  }

  [CCode (instance_pos = -1)]
  protected void del_pkg (Gtk.Button sender)
  {
    Gtk.TreeIter iter;
    Gtk.TreeSelection selection = this.widgets.tree_view_dst.get_selection ();

    if (selection.get_selected (null, out iter))
    {
      this.list_store_dst.remove (iter);
    }
  }

  public Project project
  {
    get;
    construct;
  }

  private void populate_tree_view (Gtk.TreeView tree_view)
  {
    Gtk.ListStore list_store = new Gtk.ListStore (1, typeof (string));
    list_store.set_sort_column_id (0, Gtk.SortType.ASCENDING);

    tree_view.set_model (list_store);
    Gtk.CellRendererText render = new Gtk.CellRendererText ();
    Gtk.TreeViewColumn column = new Gtk.TreeViewColumn.with_attributes ("", render, "text", 0, null);
    tree_view.append_column (column);
    tree_view.headers_visible = false;
  }

  public ProjectDialogOptions (Project project)
  {
    this.project = project;
  }

  construct
  {
    Gtk.IconView icon_view = this.widgets.icon_view;
    Gtk.ListStore list_store = new Gtk.ListStore (3, typeof (Gdk.Pixbuf),
                                                     typeof (string),
                                                     typeof (int));
    icon_view.set_model (list_store);

    Gtk.TreeIter iter;

    try
    {
      list_store.append (out iter);
      Gdk.Pixbuf icon = new Gdk.Pixbuf.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "pref-general.png"));
      list_store.set (iter, 0, icon, 1, _("General"), 2, 0, -1);
    }
    catch (Error e)
    {
      GLib.debug (e.message);
    }

    try
    {
      list_store.append (out iter);
      Gdk.Pixbuf icon = new Gdk.Pixbuf.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "pref-compiler.png"));
      list_store.set (iter, 0, icon, 1, _("Compiler"), 2, 1, -1);
    }
    catch (Error e)
    {
      GLib.debug (e.message);
    }

    try
    {
      list_store.append (out iter);
      Gdk.Pixbuf icon = new Gdk.Pixbuf.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "pref-pkg.png"));
      list_store.set (iter, 0, icon, 1, _("Packages"), 2, 2, -1);
    }
    catch (Error e)
    {
      GLib.debug (e.message);
    }

    icon_view.selection_changed += (sender) => {
      weak List<Gtk.TreePath> onglets = sender.get_selected_items ();
      if (onglets != null)
      {
        int index;
        Gtk.TreeIter iter2;

        sender.model.get_iter (out iter2, onglets.data);
        sender.model.get (iter2, 2, out index, -1);
        this.widgets.notebook.set_current_page (index);
      }
    };
    /* Set first item */
    icon_view.select_path (new Gtk.TreePath.from_string ("0"));

    this.populate_tree_view (this.widgets.tree_view_vapi);
    this.list_store_vapi = (Gtk.ListStore)this.widgets.tree_view_vapi.model;
    this.populate_tree_view (this.widgets.tree_view_src);
    this.list_store_src = (Gtk.ListStore)this.widgets.tree_view_src.model;
    this.populate_tree_view (this.widgets.tree_view_dst);
    this.list_store_dst = (Gtk.ListStore)this.widgets.tree_view_dst.model;

    this.connect_signals ("valide_project_dialog_options_");

    this.path = this.project.path;
    this.name = this.project.name;
    this.author = this.project.author;
    this.version = this.project.version;
    this.pkg = this.project.pkg;
    this.license = this.project.license;
    this.vapi_dir = this.project.vapi_dir;
    this.debug = this.project.debug;
    this.thread = this.project.thread;
    this.disable_assert = this.project.disable_assert;
    this.disable_checking = this.project.disable_checking;
    this.disable_non_null = this.project.disable_non_null;
    this.save_temps = this.project.save_temps;
    this.quiet = this.project.quiet;
    string opt = this.project.other;
    if (opt != null)
    {
      this.other = opt;
    }
    this.update_vapi ();
  }

  public int run ()
  {
    this.widgets.dialog.show_all ();
    return this.widgets.dialog.run ();
  }

  public new void destroy ()
  {
    this.widgets.dialog.destroy ();
  }
}

