/* project-dialog-remove.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Valide.ProjectDialogRemove : AbstractProjectDialogRemove
{
  Gtk.ListStore list_store;

  public Project project
  {
    get;
    construct;
  }

  public List<string> selected_files
  {
    owned get
    {
      Gtk.TreeIter iter;
      List<string> files;

      /**
       * @fixme Error if no file
       */
      this.list_store.get_iter_first (out iter);
      do
      {
        bool enable;

        this.list_store.get (iter, 0, &enable, -1);
        if (enable)
        {
          string file;

          this.list_store.get (iter, 1, &file, -1);
          files.append (file);
        }
      } while (this.list_store.iter_next (ref iter) == true);
      return files;
    }
  }

  private void toggled (Gtk.CellRendererToggle sender, string path)
  {
    Gtk.TreeIter iter;
    bool enable = false;

    this.list_store.get_iter_from_string (out iter, path);
    this.list_store.get (iter, 0, &enable, -1);
    this.list_store.set (iter, 0, !enable, -1);
  }

  public ProjectDialogRemove (Project project)
  {
    this.project = project;
  }

  construct
  {
    this.list_store = new Gtk.ListStore (2, typeof (bool), typeof (string));

    Gtk.TreeView tree_view = this.widgets.tree_view;
    tree_view.model = this.list_store;

    Gtk.CellRendererToggle render1 = new Gtk.CellRendererToggle ();
    Gtk.TreeViewColumn column = new Gtk.TreeViewColumn.with_attributes (_("Remove"), render1, "active", 0, null);
    tree_view.append_column (column);
    render1.toggled += this.toggled;

    Gtk.CellRendererText render2 = new Gtk.CellRendererText ();
    column = new Gtk.TreeViewColumn.with_attributes (_("File"), render2, "text", 1, null);
    tree_view.append_column (column);
    tree_view.headers_visible = false;

    this.widgets.label.set_text (_("Select files to remove from %s:").printf (this.project.name));
    foreach (string file in this.project.files)
    {
      Gtk.TreeIter iter;

      this.list_store.append (out iter);
      this.list_store.set (iter, 0, false, 1, file);
    }
  }

  public int run ()
  {
    this.widgets.dialog.show_all ();
    return this.widgets.dialog.run ();
  }

  public new void destroy ()
  {
    this.widgets.dialog.destroy ();
  }
}

