/* project.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Valide.Project : Object
{
  public const string[] FILE_EXT = {"vide"};

  private string _filename;
  private string working_dir;
  private Compiler compiler;
  private Executable executable;

  public signal void closed ();
  public signal void removed_file ();
  public signal void added_file ();
  public signal void options_changed ();

  private bool exist (string filename)
  {
    bool ret = false;

    foreach (string file in this.files)
    {
      if (file == filename)
      {
        ret = true;
        break;
      }
    }
    return ret;
  }

  public string filename
  {
    owned get
    {
      return this._filename;
    }
    set construct
    {
      this._filename = value;

      this.working_dir = this.path;
    }
  }

  /* General options */
  public string path
  {
    owned get
    {
      return Path.get_dirname (this.filename);
    }
  }

  public string name
  {
    owned get
    {
      return Markup.get_content (this.filename, "/project/name/text ()");
    }
    set
    {
      Markup.remove (this.filename, "/project/name");
      Markup.add_child (this.filename, "/project", "name", value);
    }
  }

  public string author
  {
    owned get
    {
      return Markup.get_content (this.filename, "/project/author/text ()");
    }
    set
    {
      Markup.remove (this.filename, "/project/author");
      Markup.add_child (this.filename, "/project", "author", value);
    }
  }

  public string version
  {
    owned get
    {
      return Markup.get_content (this.filename, "/project/version/text ()");
    }
    set
    {
      Markup.remove (this.filename, "/project/version");
      Markup.add_child (this.filename, "/project", "version", value);
    }
  }

  public string license
  {
    owned get
    {
      return Markup.get_content (this.filename, "/project/license/text ()");
    }
    set
    {
      Markup.remove (this.filename, "/project/license");
      Markup.add_child (this.filename, "/project", "license", value);
    }
  }

  public List<string> files
  {
    owned get
    {
      List<string> files_ = null;

      Xml.NodeSet* nodes = Markup.get_nodes (this.filename,
                                             "/project/files/file/text ()");
      for (int i = 0; i < nodes->length (); i++)
      {
        files_.append (nodes->item (i)->content);
      }
      return files_;
    }
  }

  /* Compiler options */
  public bool debug
  {
    get
    {
      return Markup.exist (this.filename, "/project/compiler/options/debug");
    }
    set
    {
      if (value)
      {
        if (!Markup.exist (this.filename, "/project/compiler/options/debug"))
        {
          Markup.add_child (this.filename, "/project/compiler/options", "debug");
        }
      }
      else
      {
        Markup.remove (this.filename, "/project/compiler/options/debug");
      }
    }
  }

  public bool thread
  {
    get
    {
      return Markup.exist (this.filename, "/project/compiler/options/thread");
    }
    set
    {
      if (value)
      {
        if (!Markup.exist (this.filename, "/project/compiler/options/thread"))
        {
          Markup.add_child (this.filename, "/project/compiler/options", "thread");
        }
      }
      else
      {
        Markup.remove (this.filename, "/project/compiler/options/thread");
      }
    }
  }

  public bool disable_assert
  {
    get
    {
      return Markup.exist (this.filename, "/project/compiler/options/disable_assert");
    }
    set
    {
      if (value)
      {
        if (!Markup.exist (this.filename, "/project/compiler/options/disable_assert"))
        {
          Markup.add_child (this.filename, "/project/compiler/options", "disable_assert");
        }
      }
      else
      {
        Markup.remove (this.filename, "/project/compiler/options/disable_assert");
      }
    }
  }

  public bool disable_checking
  {
    get
    {
      return Markup.exist (this.filename, "/project/compiler/options/disable_checking");
    }
    set
    {
      if (value)
      {
        if (!Markup.exist (this.filename, "/project/compiler/options/disable_checking"))
        {
          Markup.add_child (this.filename, "/project/compiler/options", "disable_checking");
        }
      }
      else
      {
        Markup.remove (this.filename, "/project/compiler/options/disable_checking");
      }
    }
  }

  public bool disable_non_null
  {
    get
    {
      return Markup.exist (this.filename, "/project/compiler/options/disable_non_null");
    }
    set
    {
      if (value)
      {
        if (!Markup.exist (this.filename, "/project/compiler/options/disable_non_null"))
        {
          Markup.add_child (this.filename, "/project/compiler/options", "disable_non_null");
        }
      }
      else
      {
        Markup.remove (this.filename, "/project/compiler/options/disable_non_null");
      }
    }
  }

  public bool save_temps
  {
    get
    {
      return Markup.exist (this.filename, "/project/compiler/options/save_temps");
    }
    set
    {
      if (value)
      {
        if (!Markup.exist (this.filename, "/project/compiler/options/save_temps"))
        {
          Markup.add_child (this.filename, "/project/compiler/options", "save_temps");
        }
      }
      else
      {
        Markup.remove (this.filename, "/project/compiler/options/save_temps");
      }
    }
  }

  public bool quiet
  {
    get
    {
      return Markup.exist (this.filename, "/project/compiler/options/quiet");
    }
    set
    {
      if (value)
      {
        if (!Markup.exist (this.filename, "/project/compiler/options/quiet"))
        {
          Markup.add_child (this.filename, "/project/compiler/options", "quiet");
        }
      }
      else
      {
        Markup.remove (this.filename, "/project/compiler/options/quiet");
      }
    }
  }

  public string other
  {
    owned get
    {
      return Markup.get_content (this.filename, "/project/compiler/options/other/text ()");
    }
    set
    {
      Markup.remove (this.filename, "/project/compiler/options/other");
      Markup.add_child (this.filename, "/project/compiler/options", "other", value);
    }
  }

  private string get_real_filename (string filename)
  {
    string path;

    if (Path.is_absolute (filename))
    {
      path = filename;
    }
    else
    {
      path = Path.build_filename (this.path, filename);
      if (!FileUtils.test (path, FileTest.EXISTS))
      {
        string xpath = "/project/files/file[contains(string(),'" + Path.DIR_SEPARATOR_S + filename + "')]/text()";
        path = Path.build_filename (this.path,
                                    Markup.get_content (this.filename, xpath));
      }
    }
    return path;
  }

  /* Packages options */
  public List<string> pkg
  {
    owned get
    {
      List<string> pkg_ = null;

      Xml.NodeSet* nodes = Markup.get_nodes (this.filename,
                                             "/project/compiler/pkg/text ()");
      for (int i = 0; i < nodes->length (); i++)
      {
        pkg_.append (nodes->item (i)->content);
      }
      return pkg_;
    }
    set
    {
      Markup.remove (this.filename, "/project/compiler/pkg");

      foreach (string p in value)
      {
        Markup.add_child (this.filename, "/project/compiler", "pkg", p);
      }
    }
  }

  public List<string> vapi_dir
  {
    owned get
    {
      List<string> vapi_ = null;

      Xml.NodeSet* nodes = Markup.get_nodes (this.filename,
                                             "/project/compiler/vapi/text ()");
      for (int i = 0; i < nodes->length (); i++)
      {
        vapi_.append (nodes->item (i)->content);
      }
      return vapi_;
    }
    set
    {
      Markup.remove (this.filename, "/project/compiler/vapi");

      foreach (string p in value)
      {
        Markup.add_child (this.filename, "/project/compiler", "vapi", p);
      }
    }
  }

  public string compiler_options
  {
    owned get
    {
      string opt = "";

      opt += "-o \"%s\"".printf (this.executable_name);

      foreach (string p in this.pkg)
      {
        opt += " --pkg=" + p;
      }

      foreach (string p in this.vapi_dir)
      {
        opt += " --vapidir=\"" + p + "\"";
      }

      if (this.debug)
      {
        opt += " -g";
      }

      if (this.thread)
      {
        opt += " --thread";
      }

      if (this.disable_assert)
      {
        opt += " --disable-assert";
      }

      if (this.disable_checking)
      {
        opt += " --disable-checking";
      }

      if (this.disable_non_null)
      {
        opt += " --disable-non-null";
      }

      if (this.save_temps)
      {
        opt += " --save-temps";
      }

      if (this.quiet)
      {
        opt += " -q";
      }

      opt += " " + this.other;

      foreach (string file in this.files)
      {
         if (file.has_suffix (".vala"))
         {
           opt += " \"%s\"".printf (file);
         }
      }

      return opt;
    }
  }

  public string executable_name
  {
    owned get
    {
      string exe_name = Path.build_filename (this.path, this.name);
  
      if (Config.OS == "win32")
      {
        exe_name += ".exe";
      }
      return exe_name;
    }
  }

  public string executable_options
  {
    owned get
    {
      string opt =  "";

      return opt;
    }
  }

  public Project (string filename)
  {
    this.filename = filename;
  }

  public void close ()
  {
    foreach (string filename in this.files)
    {
      int pos;
      string path = this.get_real_filename (filename);
      DocumentManager documents = DocumentManager.get_instance ();

      if (documents.is_open (path, out pos))
      {
        iDocument document = documents.get_nth_page (pos) as iDocument;
        documents.remove_page (document);
      }
    }
    this.closed ();
  }

  public void create_file () throws DocumentError
  {
    bool adding_file = false;
    Gtk.Dialog dialog = new Gtk.Dialog ();

    dialog.title = _("Create file");
    dialog.vbox.pack_start (new Gtk.Label (_("Would you like add file to this project?")), true, true, 0);
    dialog.add_button (Gtk.STOCK_YES, Gtk.ResponseType.YES);
    dialog.add_button (Gtk.STOCK_NO, Gtk.ResponseType.NO);
    dialog.show_all ();
    if (dialog.run () == Gtk.ResponseType.YES)
    {
      adding_file = true;
    }
    dialog.destroy ();

    iDocument doc = DocumentManager.get_instance ().create ();
    if (adding_file)
    {
      doc.save ();
      if (doc.is_save)
      {
        SList<string> files = null;

        files.append (doc.path);
        this.add_file (files);
      }
    }
  }

  public void open_file (string filename) throws DocumentError
  {
    string path = this.get_real_filename (filename);

    if (FileUtils.test (path, FileTest.EXISTS))
    {
      DocumentManager.get_instance ().create (path);
    }
    else
    {
      throw new DocumentError.BAD_URI (_("The file %s doesn't exist!"), filename);
    }
  }

  public void add_file (SList<string>? files = null)
  {
    if (files == null)
    {
      Gtk.FileChooserDialog dialog = new Gtk.FileChooserDialog (_("Add file"), null,
                                                            Gtk.FileChooserAction.OPEN,
                                                            Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                                                            Gtk.STOCK_ADD, Gtk.ResponseType.ACCEPT);

      dialog.set_current_folder (this.path);
      dialog.set_local_only (true);

      dialog.select_multiple = true;

      if (dialog.run () == Gtk.ResponseType.ACCEPT)
      {
        files = dialog.get_filenames ();
      }
      dialog.destroy ();
    }
    if (files != null)
    {
      foreach (string file in files)
      {
        string filename = Utils.get_relative_path (file, this.path);

        if (!this.exist (filename))
        {
          if (!Markup.exist (this.filename, "/project/files"))
          {
            Markup.add_child (this.filename, "/project", "files");
          }
          Markup.add_child (this.filename, "/project/files", "file", filename);
          this.added_file ();
        }
        else
        {
          message (_("File '%s' already added!"), filename);
        }
      }
    }
  }

  public void remove_file (owned List<string>? files = null)
  {
    if (files == null)
    {
      ProjectDialogRemove dialog = new ProjectDialogRemove (this);
      if (dialog.run () == Gtk.ResponseType.APPLY)
      {
        files = dialog.selected_files;
      }
      dialog.destroy ();
    }
    if (files != null)
    {
      foreach (string file in files)
      {
        Markup.remove (this.filename, "/project/files/file[.='" + file + "']");
      }
      this.removed_file ();
    }
  }

  public bool is_compiled ()
  {
    return FileUtils.test (this.executable_name, FileTest.EXISTS);
  }

  public void options ()
  {
    ProjectDialogOptions dialog = new ProjectDialogOptions (this);

    if (dialog.run () == Gtk.ResponseType.APPLY)
    {
      this.name = dialog.name;
      this.author = dialog.author;
      this.version = dialog.version;
      this.debug = dialog.debug;
      this.thread = dialog.thread;
      this.disable_assert = dialog.disable_assert;
      this.disable_checking = dialog.disable_checking;
      this.save_temps = dialog.save_temps;
      this.quiet = dialog.quiet;
      this.other = dialog.other;
      this.pkg = dialog.pkg;
      this.vapi_dir = dialog.vapi_dir;
      this.options_changed ();
    }
    dialog.destroy ();
  }

  public void compile ()
  {
    if (this.compiler == null)
    {
      this.compiler = new Compiler ();
    }
    try
    {
      this.compiler.run (this.compiler_options, this.path);
    }
    catch (Error e)
    {
      warning (e.message);
    }
  }

  public void execute ()
  {
    if (!this.is_compiled ())
    {
      this.compile ();
    }
    if (this.is_compiled ())
    {
      if (this.executable == null)
      {
        this.executable = new Executable ();
      }
      this.executable.executable = "'%s'".printf (this.executable_name);
      this.executable.run (this.executable_options, this.working_dir);
    }
  }

  /**
   * Test if a filename is a project file.
   *
   * @param filename: a filename.
   *
   * @return true if a filename is a project file.
   */
  public static bool accept_file (string filename)
  {
    bool accept = false;
    string ext = Utils.get_extension (filename);

    foreach (string e in Project.FILE_EXT)
    {
      if (e == ext)
      {
        accept = true;
        break;
      }
    }
    return accept;
  }
}


