/* executable.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_ABSTRACT_EXECUTABLE_H__
#define __LIBVALIDE_ABSTRACT_EXECUTABLE_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <libvalide/executable-manager.h>

G_BEGIN_DECLS


#define VALIDE_TYPE_ABSTRACT_EXECUTABLE (valide_abstract_executable_get_type ())
#define VALIDE_ABSTRACT_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_EXECUTABLE, ValideAbstractExecutable))
#define VALIDE_ABSTRACT_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_EXECUTABLE, ValideAbstractExecutableClass))
#define VALIDE_IS_ABSTRACT_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_EXECUTABLE))
#define VALIDE_IS_ABSTRACT_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_EXECUTABLE))
#define VALIDE_ABSTRACT_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_EXECUTABLE, ValideAbstractExecutableClass))

typedef struct _ValideAbstractExecutable ValideAbstractExecutable;
typedef struct _ValideAbstractExecutableClass ValideAbstractExecutableClass;
typedef struct _ValideAbstractExecutablePrivate ValideAbstractExecutablePrivate;

/**
 * Abstract class representing an program can launch by #ExecutableManager.
 */
struct _ValideAbstractExecutable {
	GObject parent_instance;
	ValideAbstractExecutablePrivate * priv;
};

struct _ValideAbstractExecutableClass {
	GObjectClass parent_class;
	void (*parsing) (ValideAbstractExecutable* self, ValideExecutableManager* sender, char** line);
	void (*line_activated) (ValideAbstractExecutable* self, ValideExecutableManager* sender, const char* line);
	void (*run) (ValideAbstractExecutable* self, const char* options, const char* dir);
	char* (*get_working_directory) (ValideAbstractExecutable* self);
	char* (*get_start_msg) (ValideAbstractExecutable* self);
	char* (*get_end_msg) (ValideAbstractExecutable* self);
	char* (*get_executable) (ValideAbstractExecutable* self);
	void (*set_executable) (ValideAbstractExecutable* self, const char* value);
};


void valide_abstract_executable_parsing (ValideAbstractExecutable* self, ValideExecutableManager* sender, char** line);
void valide_abstract_executable_line_activated (ValideAbstractExecutable* self, ValideExecutableManager* sender, const char* line);
void valide_abstract_executable_run (ValideAbstractExecutable* self, const char* options, const char* dir);
char* valide_abstract_executable_get_working_directory (ValideAbstractExecutable* self);
char* valide_abstract_executable_get_start_msg (ValideAbstractExecutable* self);
char* valide_abstract_executable_get_end_msg (ValideAbstractExecutable* self);
char* valide_abstract_executable_get_executable (ValideAbstractExecutable* self);
void valide_abstract_executable_set_executable (ValideAbstractExecutable* self, const char* value);
GType valide_abstract_executable_get_type (void);


G_END_DECLS

#endif
