/* abstract-document.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_ABSTRACT_TEXT_DOCUMENT_H__
#define __LIBVALIDE_ABSTRACT_TEXT_DOCUMENT_H__

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gtksourceview/gtksourceview.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <libvalide/idocument.h>
#include <libvalide/utils.h>
#include <libvalide/split-source-view.h>

G_BEGIN_DECLS


#define VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT (valide_abstract_text_document_get_type ())
#define VALIDE_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocument))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))

typedef struct _ValideAbstractTextDocument ValideAbstractTextDocument;
typedef struct _ValideAbstractTextDocumentClass ValideAbstractTextDocumentClass;
typedef struct _ValideAbstractTextDocumentPrivate ValideAbstractTextDocumentPrivate;

struct _ValideAbstractTextDocument {
	GtkVBox parent_instance;
	ValideAbstractTextDocumentPrivate * priv;
	ValideSplitSourceView* split_view;
};

struct _ValideAbstractTextDocumentClass {
	GtkVBoxClass parent_class;
	ValideSplitSourceView* (*create_source_view) (ValideAbstractTextDocument* self);
	char* (*get_contents) (ValideAbstractTextDocument* self, GError** error);
	void (*set_contents) (ValideAbstractTextDocument* self, const char* contents, GError** error);
	GdkPixbuf* (*get_icon) (ValideAbstractTextDocument* self);
};


ValideSplitSourceView* valide_abstract_text_document_create_source_view (ValideAbstractTextDocument* self);
char* valide_abstract_text_document_get_contents (ValideAbstractTextDocument* self, GError** error);
void valide_abstract_text_document_set_contents (ValideAbstractTextDocument* self, const char* contents, GError** error);
void valide_abstract_text_document_grab_focus (ValideAbstractTextDocument* self);
void valide_abstract_text_document_show (ValideAbstractTextDocument* self);
void valide_abstract_text_document_reload (ValideAbstractTextDocument* self, GError** error);
GdkPixbuf* valide_abstract_text_document_get_icon (ValideAbstractTextDocument* self);
void valide_abstract_text_document_select_range (ValideAbstractTextDocument* self, const ValidePosition* pos);
GtkSourceBuffer* valide_abstract_text_document_get_buffer (ValideAbstractTextDocument* self);
GtkSourceView* valide_abstract_text_document_get_text_view (ValideAbstractTextDocument* self);
GtkSourceView* valide_abstract_text_document_get_view1 (ValideAbstractTextDocument* self);
GtkSourceView* valide_abstract_text_document_get_view2 (ValideAbstractTextDocument* self);
gboolean valide_abstract_text_document_get_editable (ValideAbstractTextDocument* self);
void valide_abstract_text_document_set_editable (ValideAbstractTextDocument* self, gboolean value);
char* valide_abstract_text_document_get_selected_text (ValideAbstractTextDocument* self);
GType valide_abstract_text_document_get_type (void);


G_END_DECLS

#endif
