/* plugin-registrar.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_PLUGIN_REGISTRAR_H__
#define __LIBVALIDE_PLUGIN_REGISTRAR_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <libvalide/plugin.h>

G_BEGIN_DECLS


#define VALIDE_TYPE_PLUGIN_REGISTRAR (valide_plugin_registrar_get_type ())
#define VALIDE_PLUGIN_REGISTRAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrar))
#define VALIDE_PLUGIN_REGISTRAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrarClass))
#define VALIDE_IS_PLUGIN_REGISTRAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR))
#define VALIDE_IS_PLUGIN_REGISTRAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PLUGIN_REGISTRAR))
#define VALIDE_PLUGIN_REGISTRAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrarClass))

typedef struct _ValidePluginRegistrar ValidePluginRegistrar;
typedef struct _ValidePluginRegistrarClass ValidePluginRegistrarClass;
typedef struct _ValidePluginRegistrarPrivate ValidePluginRegistrarPrivate;

typedef enum  {
	VALIDE_PLUGIN_ERROR_MODULE_LOADING,
	VALIDE_PLUGIN_ERROR_MODULE_VERSION
} ValidePluginError;
#define VALIDE_PLUGIN_ERROR valide_plugin_error_quark ()
struct _ValidePluginRegistrar {
	GObject parent_instance;
	ValidePluginRegistrarPrivate * priv;
	char** authors;
	gint authors_length1;
};

struct _ValidePluginRegistrarClass {
	GObjectClass parent_class;
};


GQuark valide_plugin_error_quark (void);
ValidePluginRegistrar* valide_plugin_registrar_construct (GType object_type, const char* filename);
ValidePluginRegistrar* valide_plugin_registrar_new (const char* filename);
void valide_plugin_registrar_load (ValidePluginRegistrar* self, GError** error);
void valide_plugin_registrar_unload (ValidePluginRegistrar* self, GError** error);
void valide_plugin_registrar_preferences (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_path (ValidePluginRegistrar* self);
gboolean valide_plugin_registrar_get_is_active (ValidePluginRegistrar* self);
gboolean valide_plugin_registrar_get_is_available (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_name (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_description (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_icon_name (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_website (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_copyright (ValidePluginRegistrar* self);
gboolean valide_plugin_registrar_get_is_configurable (ValidePluginRegistrar* self);
ValidePlugin* valide_plugin_registrar_get_plugin (ValidePluginRegistrar* self);
GType valide_plugin_registrar_get_type (void);

static const char* VALIDE_PLUGIN_REGISTRAR_FILE_EXT[] = {"valide-plugin"};

G_END_DECLS

#endif
