/* project.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_PROJECT_H__
#define __LIBVALIDE_PROJECT_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS


#define VALIDE_TYPE_PROJECT (valide_project_get_type ())
#define VALIDE_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT, ValideProject))
#define VALIDE_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT, ValideProjectClass))
#define VALIDE_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT))
#define VALIDE_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT))
#define VALIDE_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT, ValideProjectClass))

typedef struct _ValideProject ValideProject;
typedef struct _ValideProjectClass ValideProjectClass;
typedef struct _ValideProjectPrivate ValideProjectPrivate;

struct _ValideProject {
	GObject parent_instance;
	ValideProjectPrivate * priv;
};

struct _ValideProjectClass {
	GObjectClass parent_class;
};


ValideProject* valide_project_construct (GType object_type, const char* filename);
ValideProject* valide_project_new (const char* filename);
void valide_project_close (ValideProject* self);
void valide_project_create_file (ValideProject* self, GError** error);
void valide_project_open_file (ValideProject* self, const char* filename, GError** error);
void valide_project_add_file (ValideProject* self, GSList* files);
void valide_project_remove_file (ValideProject* self, GList* files);
gboolean valide_project_is_compiled (ValideProject* self);
void valide_project_options (ValideProject* self);
void valide_project_compile (ValideProject* self);
void valide_project_execute (ValideProject* self);
gboolean valide_project_accept_file (const char* filename);
char* valide_project_get_filename (ValideProject* self);
void valide_project_set_filename (ValideProject* self, const char* value);
char* valide_project_get_path (ValideProject* self);
char* valide_project_get_name (ValideProject* self);
void valide_project_set_name (ValideProject* self, const char* value);
char* valide_project_get_author (ValideProject* self);
void valide_project_set_author (ValideProject* self, const char* value);
char* valide_project_get_version (ValideProject* self);
void valide_project_set_version (ValideProject* self, const char* value);
char* valide_project_get_license (ValideProject* self);
void valide_project_set_license (ValideProject* self, const char* value);
GList* valide_project_get_files (ValideProject* self);
gboolean valide_project_get_debug (ValideProject* self);
void valide_project_set_debug (ValideProject* self, gboolean value);
gboolean valide_project_get_thread (ValideProject* self);
void valide_project_set_thread (ValideProject* self, gboolean value);
gboolean valide_project_get_disable_assert (ValideProject* self);
void valide_project_set_disable_assert (ValideProject* self, gboolean value);
gboolean valide_project_get_disable_checking (ValideProject* self);
void valide_project_set_disable_checking (ValideProject* self, gboolean value);
gboolean valide_project_get_disable_non_null (ValideProject* self);
void valide_project_set_disable_non_null (ValideProject* self, gboolean value);
gboolean valide_project_get_save_temps (ValideProject* self);
void valide_project_set_save_temps (ValideProject* self, gboolean value);
gboolean valide_project_get_quiet (ValideProject* self);
void valide_project_set_quiet (ValideProject* self, gboolean value);
char* valide_project_get_other (ValideProject* self);
void valide_project_set_other (ValideProject* self, const char* value);
GList* valide_project_get_pkg (ValideProject* self);
void valide_project_set_pkg (ValideProject* self, GList* value);
GList* valide_project_get_vapi_dir (ValideProject* self);
void valide_project_set_vapi_dir (ValideProject* self, GList* value);
char* valide_project_get_compiler_options (ValideProject* self);
char* valide_project_get_executable_name (ValideProject* self);
char* valide_project_get_executable_options (ValideProject* self);
GType valide_project_get_type (void);

static const char* VALIDE_PROJECT_FILE_EXT[] = {"vide"};

G_END_DECLS

#endif
