/* utils.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_UTILS_H__
#define __LIBVALIDE_UTILS_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <libxml/xpath.h>

G_BEGIN_DECLS

typedef struct _ValidePosition ValidePosition;

#define VALIDE_TYPE_POSITION (valide_position_get_type ())

#define VALIDE_TYPE_UTILS (valide_utils_get_type ())
#define VALIDE_UTILS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UTILS, ValideUtils))
#define VALIDE_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UTILS, ValideUtilsClass))
#define VALIDE_IS_UTILS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UTILS))
#define VALIDE_IS_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UTILS))
#define VALIDE_UTILS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UTILS, ValideUtilsClass))

typedef struct _ValideUtils ValideUtils;
typedef struct _ValideUtilsClass ValideUtilsClass;
typedef struct _ValideUtilsPrivate ValideUtilsPrivate;
typedef struct _ValideParamSpecUtils ValideParamSpecUtils;

#define VALIDE_TYPE_MARKUP (valide_markup_get_type ())
#define VALIDE_MARKUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_MARKUP, ValideMarkup))
#define VALIDE_MARKUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_MARKUP, ValideMarkupClass))
#define VALIDE_IS_MARKUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_MARKUP))
#define VALIDE_IS_MARKUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_MARKUP))
#define VALIDE_MARKUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_MARKUP, ValideMarkupClass))

typedef struct _ValideMarkup ValideMarkup;
typedef struct _ValideMarkupClass ValideMarkupClass;
typedef struct _ValideMarkupPrivate ValideMarkupPrivate;
typedef struct _ValideParamSpecMarkup ValideParamSpecMarkup;

#define VALIDE_TYPE_VALA (valide_vala_get_type ())
#define VALIDE_VALA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_VALA, ValideVala))
#define VALIDE_VALA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_VALA, ValideValaClass))
#define VALIDE_IS_VALA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_VALA))
#define VALIDE_IS_VALA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_VALA))
#define VALIDE_VALA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_VALA, ValideValaClass))

typedef struct _ValideVala ValideVala;
typedef struct _ValideValaClass ValideValaClass;
typedef struct _ValideValaPrivate ValideValaPrivate;
typedef struct _ValideParamSpecVala ValideParamSpecVala;

struct _ValidePosition {
	gint start_line;
	gint end_line;
	gint start_row;
	gint end_row;
};

struct _ValideUtils {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideUtilsPrivate * priv;
};

struct _ValideUtilsClass {
	GTypeClass parent_class;
	void (*finalize) (ValideUtils *self);
};

struct _ValideParamSpecUtils {
	GParamSpec parent_instance;
};

struct _ValideMarkup {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideMarkupPrivate * priv;
};

struct _ValideMarkupClass {
	GTypeClass parent_class;
	void (*finalize) (ValideMarkup *self);
};

struct _ValideParamSpecMarkup {
	GParamSpec parent_instance;
};

struct _ValideVala {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideValaPrivate * priv;
};

struct _ValideValaClass {
	GTypeClass parent_class;
	void (*finalize) (ValideVala *self);
};

struct _ValideParamSpecVala {
	GParamSpec parent_instance;
};


ValidePosition* valide_position_dup (const ValidePosition* self);
void valide_position_free (ValidePosition* self);
GType valide_position_get_type (void);
char* valide_utils_str_replace (const char* str, const char* src, const char* dst, GError** error);
char* valide_utils_get_relative_path (const char* filename, const char* path);
char* valide_utils_get_extension (const char* filename);
guint64 valide_utils_get_mtime (const char* filename, GError** error);
void valide_utils_set_up_io_channel (gint fd, GIOCondition cond, gboolean nblock, GIOFunc func, void* func_target, GError** error);
void valide_utils_chop (char** s);
GtkButton* valide_utils_create_small_button (GtkImage* image);
GtkButton* valide_utils_create_small_button_from_stock (const char* stock_id);
GdkPixbuf* valide_utils_get_pixbuf_for_stock (const char* stock_id, GtkIconSize size);
GdkPixbuf* valide_utils_get_pixbuf_for_theme (const char* name, GtkIconSize size, GError** error);
char* valide_utils_get_mime_type (const char* filename, GError** error);
ValideUtils* valide_utils_construct (GType object_type);
ValideUtils* valide_utils_new (void);
GParamSpec* valide_param_spec_utils (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer valide_value_get_utils (const GValue* value);
void valide_value_set_utils (GValue* value, gpointer v_object);
GType valide_utils_get_type (void);
gpointer valide_utils_ref (gpointer instance);
void valide_utils_unref (gpointer instance);
void valide_markup_remove (const char* filename, const char* xpath);
void valide_markup_add_child (const char* filename, const char* xpath, const char* name, const char* content);
gboolean valide_markup_exist (const char* filename, const char* xpath);
char* valide_markup_get_content (const char* filename, const char* xpath);
xmlNodeSet* valide_markup_get_nodes (const char* filename, const char* xpath);
ValideMarkup* valide_markup_construct (GType object_type);
ValideMarkup* valide_markup_new (void);
GParamSpec* valide_param_spec_markup (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer valide_value_get_markup (const GValue* value);
void valide_value_set_markup (GValue* value, gpointer v_object);
GType valide_markup_get_type (void);
gpointer valide_markup_ref (gpointer instance);
void valide_markup_unref (gpointer instance);
GList* valide_vala_get_package_list (GList* vapi_dir);
ValideVala* valide_vala_construct (GType object_type);
ValideVala* valide_vala_new (void);
char* valide_vala_get_compiler_name (ValideVala* self);
char* valide_vala_get_prefix (ValideVala* self);
char* valide_vala_get_vapi_dir (ValideVala* self);
GParamSpec* valide_param_spec_vala (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer valide_value_get_vala (const GValue* value);
void valide_value_set_vala (GValue* value, gpointer v_object);
GType valide_vala_get_type (void);
gpointer valide_vala_ref (gpointer instance);
void valide_vala_unref (gpointer instance);


G_END_DECLS

#endif
