/* item.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_COMPLETION_ITEM_H__
#define __PLUGINS_COMPLETION_ITEM_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <vala/valamethod.h>
#include <vala/valafield.h>
#include <vala/valaproperty.h>
#include <vala/valastruct.h>
#include <vala/valaclass.h>
#include <vala/valainterface.h>
#include <vala/valasignal.h>

G_BEGIN_DECLS


#define COMPLETION_TYPE_ITEM (completion_item_get_type ())
#define COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPLETION_TYPE_ITEM, CompletionItem))
#define COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPLETION_TYPE_ITEM, CompletionItemClass))
#define COMPLETION_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPLETION_TYPE_ITEM))
#define COMPLETION_IS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPLETION_TYPE_ITEM))
#define COMPLETION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPLETION_TYPE_ITEM, CompletionItemClass))

typedef struct _CompletionItem CompletionItem;
typedef struct _CompletionItemClass CompletionItemClass;
typedef struct _CompletionItemPrivate CompletionItemPrivate;

/*
 *  Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 */
struct _CompletionItem {
	GObject parent_instance;
	CompletionItemPrivate * priv;
	char* name;
	char* type_name;
	char* info;
	gint line;
};

struct _CompletionItemClass {
	GObjectClass parent_class;
};


CompletionItem* completion_item_construct (GType object_type, const char* name);
CompletionItem* completion_item_new (const char* name);
CompletionItem* completion_item_construct_with_method (GType object_type, ValaMethod* item);
CompletionItem* completion_item_new_with_method (ValaMethod* item);
CompletionItem* completion_item_construct_with_field (GType object_type, ValaField* item);
CompletionItem* completion_item_new_with_field (ValaField* item);
CompletionItem* completion_item_construct_with_property (GType object_type, ValaProperty* item);
CompletionItem* completion_item_new_with_property (ValaProperty* item);
CompletionItem* completion_item_construct_with_struct (GType object_type, ValaStruct* item);
CompletionItem* completion_item_new_with_struct (ValaStruct* item);
CompletionItem* completion_item_construct_with_class (GType object_type, ValaClass* item);
CompletionItem* completion_item_new_with_class (ValaClass* item);
CompletionItem* completion_item_construct_with_interface (GType object_type, ValaInterface* item);
CompletionItem* completion_item_new_with_interface (ValaInterface* item);
CompletionItem* completion_item_construct_with_signal (GType object_type, ValaSignal* item);
CompletionItem* completion_item_new_with_signal (ValaSignal* item);
GType completion_item_get_type (void);


G_END_DECLS

#endif
