/* parser.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_COMPLETION_PARSER_H__
#define __PLUGINS_COMPLETION_PARSER_H__

#include <glib.h>
#include <glib-object.h>
#include <vala/valacodecontext.h>
#include <stdlib.h>
#include <string.h>
#include <plugins/completion/result.h>
#include <plugins/completion/vala-provider.h>

G_BEGIN_DECLS


#define COMPLETION_TYPE_PARSER (completion_parser_get_type ())
#define COMPLETION_PARSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPLETION_TYPE_PARSER, CompletionParser))
#define COMPLETION_PARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPLETION_TYPE_PARSER, CompletionParserClass))
#define COMPLETION_IS_PARSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPLETION_TYPE_PARSER))
#define COMPLETION_IS_PARSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPLETION_TYPE_PARSER))
#define COMPLETION_PARSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPLETION_TYPE_PARSER, CompletionParserClass))

typedef struct _CompletionParser CompletionParser;
typedef struct _CompletionParserClass CompletionParserClass;
typedef struct _CompletionParserPrivate CompletionParserPrivate;

struct _CompletionParser {
	GObject parent_instance;
	CompletionParserPrivate * priv;
};

struct _CompletionParserClass {
	GObjectClass parent_class;
};


void completion_parser_parse_context (CompletionParser* self, ValaCodeContext* context);
gboolean completion_parser_add_vapidir (CompletionParser* self, const char* uri);
gboolean completion_parser_remove_vapidir (CompletionParser* self, const char* uri);
gboolean completion_parser_add_package (CompletionParser* self, const char* name);
gboolean completion_parser_remove_package (CompletionParser* self, const char* name);
gboolean completion_parser_add_source (CompletionParser* self, const char* uri);
gboolean completion_parser_remove_source (CompletionParser* self, const char* uri);
CompletionResult* completion_parser_get_completions_for_name (CompletionParser* self, CompletionFilterOptions* options, const char* symbolname, const char* sourcefile, GError** error);
CompletionParser* completion_parser_construct (GType object_type);
CompletionParser* completion_parser_new (void);
GType completion_parser_get_type (void);


G_END_DECLS

#endif
