/* documentwords-provider.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_COMPLETION_SOURCE_COMPLETION_DOCUMENTWORDS_PROVIDER_H__
#define __PLUGINS_COMPLETION_SOURCE_COMPLETION_DOCUMENTWORDS_PROVIDER_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <plugins/completion/source-completion/provider.h>
#include <plugins/completion/source-completion/trigger.h>

G_BEGIN_DECLS


#define SOURCE_COMPLETION_TYPE_DOCUMENTWORDS_PROVIDER (source_completion_documentwords_provider_get_type ())
#define SOURCE_COMPLETION_DOCUMENTWORDS_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SOURCE_COMPLETION_TYPE_DOCUMENTWORDS_PROVIDER, SourceCompletionDocumentwordsProvider))
#define SOURCE_COMPLETION_DOCUMENTWORDS_PROVIDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SOURCE_COMPLETION_TYPE_DOCUMENTWORDS_PROVIDER, SourceCompletionDocumentwordsProviderClass))
#define SOURCE_COMPLETION_IS_DOCUMENTWORDS_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SOURCE_COMPLETION_TYPE_DOCUMENTWORDS_PROVIDER))
#define SOURCE_COMPLETION_IS_DOCUMENTWORDS_PROVIDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SOURCE_COMPLETION_TYPE_DOCUMENTWORDS_PROVIDER))
#define SOURCE_COMPLETION_DOCUMENTWORDS_PROVIDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SOURCE_COMPLETION_TYPE_DOCUMENTWORDS_PROVIDER, SourceCompletionDocumentwordsProviderClass))

typedef struct _SourceCompletionDocumentwordsProvider SourceCompletionDocumentwordsProvider;
typedef struct _SourceCompletionDocumentwordsProviderClass SourceCompletionDocumentwordsProviderClass;
typedef struct _SourceCompletionDocumentwordsProviderPrivate SourceCompletionDocumentwordsProviderPrivate;

#define SOURCE_COMPLETION_DOCUMENTWORDS_PROVIDER_TYPE_SORT_TYPE (source_completion_documentwords_provider_sort_type_get_type ())

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2007 - Jess Barbero
 *
 * rev 163
 */
struct _SourceCompletionDocumentwordsProvider {
	GObject parent_instance;
	SourceCompletionDocumentwordsProviderPrivate * priv;
};

struct _SourceCompletionDocumentwordsProviderClass {
	GObjectClass parent_class;
	char* (*get_name) (SourceCompletionDocumentwordsProvider* self);
	GList* (*get_proposals) (SourceCompletionDocumentwordsProvider* self, SourceCompletionTrigger* trigger);
	void (*finish) (SourceCompletionDocumentwordsProvider* self);
};

typedef enum  {
	SOURCE_COMPLETION_DOCUMENTWORDS_PROVIDER_SORT_TYPE_NONE,
	SOURCE_COMPLETION_DOCUMENTWORDS_PROVIDER_SORT_TYPE_BY_LENGTH
} SourceCompletionDocumentwordsProviderSortType;


GType source_completion_documentwords_provider_sort_type_get_type (void);
gboolean source_completion_documentwords_provider_pred_is_separator (SourceCompletionDocumentwordsProvider* self, gunichar ch);
gint source_completion_documentwords_provider_utf8_len_compare (void* a, void* b);
char* source_completion_documentwords_provider_get_name (SourceCompletionDocumentwordsProvider* self);
GList* source_completion_documentwords_provider_get_proposals (SourceCompletionDocumentwordsProvider* self, SourceCompletionTrigger* trigger);
void source_completion_documentwords_provider_finish (SourceCompletionDocumentwordsProvider* self);
SourceCompletionDocumentwordsProvider* source_completion_documentwords_provider_construct (GType object_type, GtkTextView* view);
SourceCompletionDocumentwordsProvider* source_completion_documentwords_provider_new (GtkTextView* view);
GtkTextView* source_completion_documentwords_provider_get_view (SourceCompletionDocumentwordsProvider* self);
SourceCompletionDocumentwordsProviderSortType source_completion_documentwords_provider_get_sort_type (SourceCompletionDocumentwordsProvider* self);
void source_completion_documentwords_provider_set_sort_type (SourceCompletionDocumentwordsProvider* self, SourceCompletionDocumentwordsProviderSortType value);
GType source_completion_documentwords_provider_get_type (void);


G_END_DECLS

#endif
