/* manager.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_COMPLETION_SOURCE_COMPLETION_MANAGER_H__
#define __PLUGINS_COMPLETION_SOURCE_COMPLETION_MANAGER_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <plugins/completion/source-completion/popup.h>
#include <plugins/completion/source-completion/provider.h>
#include <plugins/completion/source-completion/trigger.h>

G_BEGIN_DECLS


#define SOURCE_COMPLETION_TYPE_MANAGER (source_completion_manager_get_type ())
#define SOURCE_COMPLETION_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SOURCE_COMPLETION_TYPE_MANAGER, SourceCompletionManager))
#define SOURCE_COMPLETION_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SOURCE_COMPLETION_TYPE_MANAGER, SourceCompletionManagerClass))
#define SOURCE_COMPLETION_IS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SOURCE_COMPLETION_TYPE_MANAGER))
#define SOURCE_COMPLETION_IS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SOURCE_COMPLETION_TYPE_MANAGER))
#define SOURCE_COMPLETION_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SOURCE_COMPLETION_TYPE_MANAGER, SourceCompletionManagerClass))

typedef struct _SourceCompletionManager SourceCompletionManager;
typedef struct _SourceCompletionManagerClass SourceCompletionManagerClass;
typedef struct _SourceCompletionManagerPrivate SourceCompletionManagerPrivate;

#define SOURCE_COMPLETION_MANAGER_TYPE_EVENT_OPTIONS (source_completion_manager_event_options_get_type ())
#define SOURCE_COMPLETION_MANAGER_EVENT_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SOURCE_COMPLETION_MANAGER_TYPE_EVENT_OPTIONS, SourceCompletionManagerEventOptions))
#define SOURCE_COMPLETION_MANAGER_EVENT_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SOURCE_COMPLETION_MANAGER_TYPE_EVENT_OPTIONS, SourceCompletionManagerEventOptionsClass))
#define SOURCE_COMPLETION_MANAGER_IS_EVENT_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SOURCE_COMPLETION_MANAGER_TYPE_EVENT_OPTIONS))
#define SOURCE_COMPLETION_MANAGER_IS_EVENT_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SOURCE_COMPLETION_MANAGER_TYPE_EVENT_OPTIONS))
#define SOURCE_COMPLETION_MANAGER_EVENT_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SOURCE_COMPLETION_MANAGER_TYPE_EVENT_OPTIONS, SourceCompletionManagerEventOptionsClass))

typedef struct _SourceCompletionManagerEventOptions SourceCompletionManagerEventOptions;
typedef struct _SourceCompletionManagerEventOptionsClass SourceCompletionManagerEventOptionsClass;
typedef struct _SourceCompletionManagerEventOptionsPrivate SourceCompletionManagerEventOptionsPrivate;
typedef struct _SourceCompletionManagerParamSpecEventOptions SourceCompletionManagerParamSpecEventOptions;

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2007 - Chuchiperriman
 *
 * rev 163
 */
struct _SourceCompletionManager {
	GObject parent_instance;
	SourceCompletionManagerPrivate * priv;
};

struct _SourceCompletionManagerClass {
	GObjectClass parent_class;
};

struct _SourceCompletionManagerEventOptions {
	GTypeInstance parent_instance;
	volatile int ref_count;
	SourceCompletionManagerEventOptionsPrivate * priv;
	SourceCompletionPopupPositionType position_type;
	SourceCompletionPopupFilterType filter_type;
	char* filter_text;
	gboolean autoselect;
	gboolean show_bottom_bar;
};

struct _SourceCompletionManagerEventOptionsClass {
	GTypeClass parent_class;
	void (*finalize) (SourceCompletionManagerEventOptions *self);
};

struct _SourceCompletionManagerParamSpecEventOptions {
	GParamSpec parent_instance;
};


#define SOURCE_COMPLETION_MANAGER_USER_REQUEST_TRIGGER_NAME "user-request"
SourceCompletionManager* source_completion_manager_get_instance (GtkTextView* view);
gboolean source_completion_manager_register_provider (SourceCompletionManager* self, SourceCompletionProvider* provider, const char* trigger_name);
gboolean source_completion_manager_unregister_provider (SourceCompletionManager* self, SourceCompletionProvider* provider, const char* trigger_name);
GtkTextView* source_completion_manager_get_view (SourceCompletionManager* self);
void source_completion_manager_update_event_options (SourceCompletionManager* self, SourceCompletionManagerEventOptions* options);
void source_completion_manager_trigger_event_with_opts (SourceCompletionManager* self, const char* trigger_name, SourceCompletionManagerEventOptions* options, void* event_data);
void source_completion_manager_trigger_event (SourceCompletionManager* self, const char* trigger_name, void* event_data);
gboolean source_completion_manager_is_visible (SourceCompletionManager* self);
SourceCompletionManager* source_completion_manager_get_from_view (GtkTextView* view);
SourceCompletionProvider* source_completion_manager_get_provider (SourceCompletionManager* self, const char* provider_name);
void source_completion_manager_register_trigger (SourceCompletionManager* self, SourceCompletionTrigger* trigger);
void source_completion_manager_unregister_trigger (SourceCompletionManager* self, SourceCompletionTrigger* trigger);
SourceCompletionTrigger* source_completion_manager_get_trigger (SourceCompletionManager* self, const char* trigger_name);
void source_completion_manager_activate (SourceCompletionManager* self);
void source_completion_manager_deactivate (SourceCompletionManager* self);
void source_completion_manager_finish_completion (SourceCompletionManager* self);
SourceCompletionTrigger* source_completion_manager_get_active_trigger (SourceCompletionManager* self);
void source_completion_manager_set_current_info (SourceCompletionManager* self, const char* info);
void source_completion_manager_get_current_event_options (SourceCompletionManager* self, SourceCompletionManagerEventOptions** options);
char* source_completion_manager_get_info_keys (SourceCompletionManager* self);
void source_completion_manager_set_info_keys (SourceCompletionManager* self, const char* value);
char* source_completion_manager_get_next_page_keys (SourceCompletionManager* self);
void source_completion_manager_set_next_page_keys (SourceCompletionManager* self, const char* value);
char* source_completion_manager_get_previous_page_keys (SourceCompletionManager* self);
void source_completion_manager_set_previous_page_keys (SourceCompletionManager* self, const char* value);
SourceCompletionManagerEventOptions* source_completion_manager_event_options_construct (GType object_type);
SourceCompletionManagerEventOptions* source_completion_manager_event_options_new (void);
GParamSpec* source_completion_manager_param_spec_event_options (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer source_completion_manager_value_get_event_options (const GValue* value);
void source_completion_manager_value_set_event_options (GValue* value, gpointer v_object);
GType source_completion_manager_event_options_get_type (void);
gpointer source_completion_manager_event_options_ref (gpointer instance);
void source_completion_manager_event_options_unref (gpointer instance);
GType source_completion_manager_get_type (void);


G_END_DECLS

#endif
