/* vala-provider.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_COMPLETION_VALA_PROVIDER_H__
#define __PLUGINS_COMPLETION_VALA_PROVIDER_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <plugins/completion/source-completion/provider.h>
#include <plugins/completion/source-completion/trigger.h>

G_BEGIN_DECLS


#define COMPLETION_TYPE_FILTER_OPTIONS (completion_filter_options_get_type ())
#define COMPLETION_FILTER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPLETION_TYPE_FILTER_OPTIONS, CompletionFilterOptions))
#define COMPLETION_FILTER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPLETION_TYPE_FILTER_OPTIONS, CompletionFilterOptionsClass))
#define COMPLETION_IS_FILTER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPLETION_TYPE_FILTER_OPTIONS))
#define COMPLETION_IS_FILTER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPLETION_TYPE_FILTER_OPTIONS))
#define COMPLETION_FILTER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPLETION_TYPE_FILTER_OPTIONS, CompletionFilterOptionsClass))

typedef struct _CompletionFilterOptions CompletionFilterOptions;
typedef struct _CompletionFilterOptionsClass CompletionFilterOptionsClass;
typedef struct _CompletionFilterOptionsPrivate CompletionFilterOptionsPrivate;
typedef struct _CompletionParamSpecFilterOptions CompletionParamSpecFilterOptions;

#define COMPLETION_TYPE_VALA_PROVIDER (completion_vala_provider_get_type ())
#define COMPLETION_VALA_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPLETION_TYPE_VALA_PROVIDER, CompletionValaProvider))
#define COMPLETION_VALA_PROVIDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPLETION_TYPE_VALA_PROVIDER, CompletionValaProviderClass))
#define COMPLETION_IS_VALA_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPLETION_TYPE_VALA_PROVIDER))
#define COMPLETION_IS_VALA_PROVIDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPLETION_TYPE_VALA_PROVIDER))
#define COMPLETION_VALA_PROVIDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPLETION_TYPE_VALA_PROVIDER, CompletionValaProviderClass))

typedef struct _CompletionValaProvider CompletionValaProvider;
typedef struct _CompletionValaProviderClass CompletionValaProviderClass;
typedef struct _CompletionValaProviderPrivate CompletionValaProviderPrivate;

/*
 *  Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 */
struct _CompletionFilterOptions {
	GTypeInstance parent_instance;
	volatile int ref_count;
	CompletionFilterOptionsPrivate * priv;
	gboolean static_symbols;
	gboolean interface_symbols;
	gboolean only_constructors;
	gboolean private_symbols;
	gboolean public_symbols;
	gboolean protected_symbols;
	gboolean internal_symbols;
	char* exclude_type;
};

struct _CompletionFilterOptionsClass {
	GTypeClass parent_class;
	void (*finalize) (CompletionFilterOptions *self);
};

struct _CompletionParamSpecFilterOptions {
	GParamSpec parent_instance;
};

struct _CompletionValaProvider {
	GObject parent_instance;
	CompletionValaProviderPrivate * priv;
};

struct _CompletionValaProviderClass {
	GObjectClass parent_class;
};


void completion_filter_options_public_only (CompletionFilterOptions* self);
CompletionFilterOptions* completion_filter_options_construct (GType object_type);
CompletionFilterOptions* completion_filter_options_new (void);
GParamSpec* completion_param_spec_filter_options (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer completion_value_get_filter_options (const GValue* value);
void completion_value_set_filter_options (GValue* value, gpointer v_object);
GType completion_filter_options_get_type (void);
gpointer completion_filter_options_ref (gpointer instance);
void completion_filter_options_unref (gpointer instance);
CompletionValaProvider* completion_vala_provider_construct (GType object_type, GtkTextView* text_view);
CompletionValaProvider* completion_vala_provider_new (GtkTextView* text_view);
GtkTextView* completion_vala_provider_get_text_view (CompletionValaProvider* self);
GType completion_vala_provider_get_type (void);


G_END_DECLS

#endif
