/* symbol-browser.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_SYMBOL_SYMBOL_H__
#define __PLUGINS_SYMBOL_SYMBOL_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libvalide/abstract-text-document.h>
#include <libvalide/plugin.h>

G_BEGIN_DECLS


#define TYPE_SYMBOL (symbol_get_type ())
#define SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SYMBOL, Symbol))
#define SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SYMBOL, SymbolClass))
#define IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SYMBOL))
#define IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SYMBOL))
#define SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SYMBOL, SymbolClass))

typedef struct _Symbol Symbol;
typedef struct _SymbolClass SymbolClass;
typedef struct _SymbolPrivate SymbolPrivate;
typedef struct _ParamSpecSymbol ParamSpecSymbol;

#define TYPE_SYMBOL_BROWSER (symbol_browser_get_type ())
#define SYMBOL_BROWSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SYMBOL_BROWSER, SymbolBrowser))
#define SYMBOL_BROWSER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SYMBOL_BROWSER, SymbolBrowserClass))
#define IS_SYMBOL_BROWSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SYMBOL_BROWSER))
#define IS_SYMBOL_BROWSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SYMBOL_BROWSER))
#define SYMBOL_BROWSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SYMBOL_BROWSER, SymbolBrowserClass))

typedef struct _SymbolBrowser SymbolBrowser;
typedef struct _SymbolBrowserClass SymbolBrowserClass;
typedef struct _SymbolBrowserPrivate SymbolBrowserPrivate;

/*
 * This code is re-writing in the vala language of GeditSymbolBrowserPlugin
 *
 * Copyright (C) 2007 - Micah Carrick
 
 just used to keep the population code readable */
struct _Symbol {
	GTypeInstance parent_instance;
	volatile int ref_count;
	SymbolPrivate * priv;
	char* name;
	char* type;
	char* file;
	char* language;
	gint line;
	GtkTreeIter iter;
	Symbol* root;
	ValideAbstractTextDocument* document;
};

struct _SymbolClass {
	GTypeClass parent_class;
	void (*finalize) (Symbol *self);
};

struct _ParamSpecSymbol {
	GParamSpec parent_instance;
};

struct _SymbolBrowser {
	ValidePlugin parent_instance;
	SymbolBrowserPrivate * priv;
};

struct _SymbolBrowserClass {
	ValidePluginClass parent_class;
};


Symbol* symbol_construct (GType object_type);
Symbol* symbol_new (void);
GParamSpec* param_spec_symbol (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer value_get_symbol (const GValue* value);
void value_set_symbol (GValue* value, gpointer v_object);
GType symbol_get_type (void);
gpointer symbol_ref (gpointer instance);
void symbol_unref (gpointer instance);
SymbolBrowser* symbol_browser_construct (GType object_type, const char* path);
SymbolBrowser* symbol_browser_new (const char* path);
GtkWidget* symbol_browser_create_configure_dialog (SymbolBrowser* self);
GType symbol_browser_get_type (void);
GType register_plugin (void);
GtkWidget* create_configure_dialog (SymbolBrowser* self);


G_END_DECLS

#endif
