/* info.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2007 - Chuchiperriman
 *
 * rev 163
 */

public class SourceCompletion.Info : Gtk.Window
{
  private Gtk.Label label;
  private InfoType type;
  private bool adjust_height;
  private bool adjust_width;
  private int max_height;
  private int max_width;
  private const int WINDOW_WIDTH=350;
  private const int WINDOW_HEIGHT=200;

  public signal void info_type_changed (InfoType type);

  public enum InfoType
  {
    VIEW_SORT,
    VIEW_EXTENDED
  }

  [Callback]
  private bool focus_out_event_cb (Gtk.Widget widget, Gdk.EventFocus event)
  {
    widget.hide();
    return false;
  }

  public void show ()
  {
    /* TODO Control the "+30" for the scrollbars */
    string text = this.label.get_text ();
    if (text == null || text == "")
      return;

    Gtk.Requisition req;
    int w, h;
    this.label.size_request (out req);
  
    if (this.adjust_height)
      h = req.height > this.max_height ? this.max_height : req.height + 30;
    else
      h = this.WINDOW_HEIGHT;
  
    if (this.adjust_width)
      w = req.width > this.max_width ? this.max_width : req.width + 30;
    else
      w = this.WINDOW_WIDTH;

    this.resize (w, h);
  
    base.show_all ();
  }

  public void hide()
  {
    this.label.set_label ("");
    base.hide ();
  }

  construct
  {
    this.adjust_height = false;
    this.adjust_width = false;
    this.max_height = this.WINDOW_HEIGHT;
    this.max_width = this.WINDOW_WIDTH;
    this.set_default_size (this.WINDOW_WIDTH, this.WINDOW_HEIGHT);
    this.set_decorated (false);
    Gtk.ScrolledWindow info_scroll = new Gtk.ScrolledWindow (null, null);
    info_scroll.has_default = false;
    info_scroll.can_focus = false;
    info_scroll.has_focus = false;
    info_scroll.is_focus = false;
    info_scroll.set_policy (Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC);
    this.label = new Gtk.Label (null);
    this.label.has_default = false;
    this.label.can_focus = false;
    this.label.has_focus = false;
    this.label.is_focus = false;
    info_scroll.add_with_viewport (this.label);
    this.set_border_width (1);
    this.add (info_scroll);

    this.set_type_hint (Gdk.WindowTypeHint.NORMAL);
    this.set_skip_pager_hint (true);
    this.set_skip_taskbar_hint (true);
    this.set_focus_on_map (false);
    this.set_accept_focus (false);
    this.has_default = false;
    this.can_focus = false;
    ((Gtk.Widget)this).has_focus = false;
    this.is_focus = false;
  }

  public Info ()
  {
    ((Gtk.Window)this).type = Gtk.WindowType.POPUP;
  }

  public void move_to_cursor (Gtk.TextView view)
  {
    int x, y;
    Utils.get_window_position_in_cursor (this, view, out x, out y);
    this.move (x, y);
  }

  public void set_markup (string markup)
  {
    this.label.set_markup (markup);
  }

  public void set_adjust_height (bool adjust, int max_height)
  {
    // TODO Control if max_height > screen height
    this.adjust_height = adjust;
    if (max_height > 0)
      this.max_height = max_height;
  }

  public void set_adjust_width (bool adjust, int max_width)
  {
    // TODO Control if max_width > screen width
    this.adjust_width = adjust;
    if (max_width > 0)
      this.max_width = max_width;
  }

  public void set_info_type (InfoType type)
  {
    if (this.type != type)
    {
      this.type = type;
      this.info_type_changed (type);
    }
  }
}

