/* provider-file.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2008 - perriman
 *
 * rev 163
 */

public class SourceCompletion.ProviderFile : Object, Provider
{
  private File file_uri;
  private Gdk.Pixbuf icon;
  private List current_props;
  private string current_word;
  private const string FILE_NAME = "GscProviderFile";
  private const string ICON_FILE = "default.png";

  public Gtk.TextView view
  {
    get;
    construct;
  }

  private void table_foreach (void* key, void* value)
  {
    Proposal prop = null;
  
    if (Utils.is_valid_word (this.current_word, (string)key))
    {
      this.current_props.append (new Proposal ((string)key, null, this.icon));
    }
  }

  private weak List get_words (string buffer)
  {
    this.current_word = Utils.get_last_word_cleaned (this.view);
    HashTable<string, int> table = new HashTable<string, int> (str_hash, str_equal);
    string ini = null;
    string cur = buffer;
    unichar c;
    int count = 0;
    int words = 0;

    while (true)
    {
      ini = cur;
      count = 0;
      c = cur.get_char ();
      while (!Utils.char_is_separator (c))
      {
        count++;
        cur = cur.next_char ();
        c = cur.get_char ();
      }
      if (count > 0)
      {
        table.insert (ini.ndup (count), 0);
        words++;
      }

      if (cur.get_char () == '\0')
      {
        break;
      }

      cur = cur.next_char ();
    }

    this.current_props = null;
    table.for_each (this.table_foreach);

    table = null;
    return this.current_props.first ();
  }

  public virtual string get_name ()
  {
    return this.FILE_NAME;
  }

  public virtual List? get_proposals (Trigger trigger)
  {
    FileInfo info;
    try
    {
      info = this.file_uri.query_info (FILE_ATTRIBUTE_STANDARD_SIZE, FileQueryInfoFlags.NONE, null);
    }
    catch (Error e)
    {
      warning (_("The file does not exist"));
      return null;
    }

    ulong nchars;
    string buffer = null;
    FileUtils.get_contents (this.file_uri.get_path (), out buffer, out nchars);

    if (nchars > 0)
    {
      if (!buffer.validate ((long)nchars))
      {
        size_t bytes_read;
        string converted_text;

        converted_text = buffer.locale_to_utf8 ((long)nchars, out bytes_read, out nchars);
      /* TODO support for other encodings
      converted_text = g_convert (buffer,
               nchars,
               "UTF-8",
               "ISO-8859-1",
               &bytes_read,
               &new_len,
               NULL);
      */
      
        if (converted_text == null)
        {
          warning (_("The file does not look like a text file or the file encoding is not supported."));
          return null;
        }
        buffer = converted_text;
      }
    }
    return this.get_words (buffer).copy ();
  }

  public virtual void finish ()
  {
  }

  construct
  {
    this.file_uri = null;
    this.icon = new Gdk.Pixbuf.from_file (Path.build_filename (Config.PACKAGE_PLUGIN_DIR, "completion", "pixmaps", this.ICON_FILE));
    this.current_props = null;
  }

  public ProviderFile (Gtk.TextView view)
  {
    this.view = view;
  }

  public void set_file (string file)
  {
    this.file_uri = File.new_for_path (file); 
  }
}

