/* vala-provider.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/*
 *  Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 */

public class Completion.FilterOptions
{
  public bool static_symbols = true;
  public bool interface_symbols = true;
  public bool only_constructors = false;

  public bool private_symbols = true;
  public bool public_symbols = true;
  public bool protected_symbols = true;
  public bool internal_symbols = true;

  public string exclude_type = null;

  public void public_only ()
  {
    this.public_symbols = true;
    this.private_symbols = false;
    this.protected_symbols = false;
    this.internal_symbols = false;
  }
}

public class Completion.ValaProvider : Object, SourceCompletion.Provider
{
  private SourceBuffer _sb = null;
  private List<SourceCompletion.Proposal> list;
  private int prealloc_index = 0;
  private Gdk.Pixbuf icon_generic;
  private Gdk.Pixbuf icon_field;
  private Gdk.Pixbuf icon_method;
  private Gdk.Pixbuf icon_class;
  private Gdk.Pixbuf icon_struct;
  private Gdk.Pixbuf icon_property;
  private Gdk.Pixbuf icon_signal;
  private Gdk.Pixbuf icon_iface;
  private Gdk.Pixbuf icon_const;
  private Gdk.Pixbuf icon_enum;
  private Gdk.Pixbuf icon_namespace;
  private const int prealloc_count = 500;
  private bool cache_building = false;
  private SourceCompletion.Trigger last_trigger = null;
  private Parser parser;

  public Gtk.TextView text_view
  {
    get;
    construct;
  }

  private static int proposal_sort (void* a, void* b)
  {
    SourceCompletion.Proposal pa = (SourceCompletion.Proposal)a;
    SourceCompletion.Proposal pb = (SourceCompletion.Proposal)b;

    return strcmp (pa.label, pb.label);
  }

  private void append_symbols (Gee.List<Item> symbols, Gdk.Pixbuf icon)
  {
    foreach (Item symbol in symbols)
    {
      string name = (symbol.name != null ? symbol.name : "<null>");
      string info = (symbol.info != null ? symbol.info : "");

      SourceCompletion.Proposal proposal = new SourceCompletion.Proposal (name, info, icon);
      this.list.append (proposal);
    }
    //sort list
    this.list.sort (this.proposal_sort);
  }

  private void transform_result (Result? result)
  {
    Timer timer = new Timer ();

    this.prealloc_index = 0;
    this.list = new List<SourceCompletion.Proposal> ();

    if (result != null && !result.is_empty)
    {
      if (result.fields.size > 0)
      {
        this.append_symbols (result.fields, this.icon_field);
      }
      if (result.properties.size > 0)
      {
        this.append_symbols (result.properties, this.icon_property);
      }
      if (result.methods.size > 0)
      {
        this.append_symbols (result.methods, this.icon_method);
      }
      if (result.signals.size > 0)
      {
        this.append_symbols (result.signals, this.icon_signal);
      }
      if (result.classes.size > 0)
      {
        this.append_symbols (result.classes, this.icon_class);
      }
      if (result.interfaces.size > 0)
      {
        this.append_symbols (result.interfaces, this.icon_iface);
      }
      if (result.structs.size > 0)
      {
        this.append_symbols (result.structs, this.icon_struct);
      }
      if (result.enums.size > 0)
      {
        this.append_symbols (result.enums, this.icon_enum);
      }
      if (result.constants.size > 0)
      {
        this.append_symbols (result.constants, this.icon_const);
      }
      if (result.namespaces.size > 0)
      {
        this.append_symbols (result.namespaces, this.icon_namespace);
      }
      if (result.others.size > 0)
      {
        this.append_symbols (result.others, this.icon_generic);
      }
    }
    timer.stop ();
    GLib.debug ("     TRANSFORM TOTAL TIME ELAPSED: %f", timer.elapsed ());
  }

  private void parse_current_line (bool skip_leading_spaces, out string word, out string line, out int lineno, out int colno)
  {
    StringBuilder tmp = new StringBuilder ();

    weak Gtk.TextBuffer buffer = this.text_view.buffer;
    weak Gtk.TextMark mark = buffer.get_insert ();
    Gtk.TextIter end;
    Gtk.TextIter start;

    buffer.get_iter_at_mark (out start, mark);

    lineno = start.get_line ();
    colno = start.get_line_offset ();
    word = "";
    line = "";

    end = start;
    start.set_line_offset (0);
    line = start.get_text (end);

    if (colno > 1)
    {
      start = end;
      while (true)
      {
        start.backward_char ();
        if (start.starts_line ())
        {
          break;
        }

        unichar ch = start.get_char ();
        if (skip_leading_spaces && ch == ' ')
        {
          if (skip_leading_spaces && ch == ' ')
          {
            while (true)
            {
              start.backward_char ();
              if (start.starts_line ())
              {
                break;
              }
              ch = start.get_char ();
              if (ch != ' ')
              {
                break;
              }
            }
            start.forward_char ();
          }
          else if (ch == ')')
          {
            //tmp.append_unichar (ch);
            //find where the method invocation begin
            while (true)
            {
              start.backward_char ();
              if (start.starts_line ())
              {
                break;
              }

              ch = start.get_char ();
              if (ch == '(')
              {
                //tmp.append_unichar (ch);
                break;
              }
            }
            //skip spaces
            if (!start.starts_line ())
            {
              while (true)
              {
                start.backward_char ();
                if (start.starts_line ())
                {
                  break;
                }

                ch = start.get_char ();
                if (ch != ' ')
                {
                  break;
                }
              }
            }
            if (!start.starts_line ())
            {
              tmp.append_unichar (ch);
            }
          }
          else if (ch == '\"')
          {
            //find where the string literal begin
            tmp.append_unichar (ch);
            while (true)
            {
              start.backward_char ();
              if (start.starts_line ())
              {
                break;
              }

              ch = start.get_char ();
              tmp.append_unichar (ch);
              if (ch == '\"')
              {
                break;
              }
            }
          }
          else if (!(ch.isalnum () || ch == '.' || ch == '_'))
          {
            break;
          }
          else
          {
            tmp.append_unichar (ch);
          }
          skip_leading_spaces = false;
        }
        word = tmp.str.reverse ();
      }
    }
  }

  private Result? find_proposals ()
  {
    Result result = null;
    string line, word;
    int lineno, colno;

    this.parse_current_line (false, out word, out line, out lineno, out colno);

    if (word == null || word == "")
    {
      try
      {
        string typename = null;
        Timer timer = new Timer ();
        if (word.has_prefix ("\"") && word.has_suffix ("\""))
        {
          typename = "string";
        }
        else
        {
          //typename = this.parser.get_datatype_name_for_name (word, this.sb.name, lineno + 1, colno);
        }

        FilterOptions options = new FilterOptions ();
        options.public_only ();
        if (line.str ("= new ") != null || line.str ("=new ") != null)
        {
          options.only_constructors = true;
        }
        if (typename != null)
        {
          debug ("datatype '%s' for: %s", typename, word);
          options.static_symbols = false;
          if (word == "this")
          {
            options.private_symbols = true;
            options.protected_symbols = true;
          }
          else if (word == "base")
          {
            options.protected_symbols = true;
          }
          options.exclude_type = typename;						

          timer.start ();
          result = this.parser.get_completions_for_name (options, typename, _sb.name);
          timer.stop ();
          debug ("find_by_name: %f", timer.elapsed ());
        }
        else
        {
          debug ("data type not found for: %s", word);
          if (word.has_prefix ("this.") == false && word.has_prefix ("base.") == false)
          {
            options.static_symbols = true;
            options.interface_symbols = false;
            timer.start ();
            result = this.parser.get_completions_for_name (options, word, _sb.name);
            timer.stop ();
            debug ("find_by_name (static): %f, count %d", timer.elapsed (), result.count);
          }
        }
      }
      catch (Error err)
      {
        warning ("%s", err.message);
      }
    }
    return result;
  }

  private inline string get_image_path (string name)
  {
    return Path.build_filename (Config.PACKAGE_PLUGIN_DIR, "completion", "pixmaps", name);
  }

  construct
  {
    try
    {
      this.icon_generic = Gtk.IconTheme.get_default().load_icon (Gtk.STOCK_FILE, 16, Gtk.IconLookupFlags.GENERIC_FALLBACK);
      this.icon_field = new Gdk.Pixbuf.from_file (this.get_image_path ("element-field-16.png"));
      this.icon_method = new Gdk.Pixbuf.from_file (this.get_image_path ("element-method-16.png"));
      this.icon_class = new Gdk.Pixbuf.from_file (this.get_image_path ("element-class-16.png"));
      this.icon_struct = new Gdk.Pixbuf.from_file (this.get_image_path ("element-structure-16.png"));
      this.icon_property = new Gdk.Pixbuf.from_file (this.get_image_path ("element-property-16.png"));
      this.icon_signal = new Gdk.Pixbuf.from_file (this.get_image_path ("element-event-16.png"));
      this.icon_iface = new Gdk.Pixbuf.from_file (this.get_image_path ("element-interface-16.png"));
      this.icon_enum = new Gdk.Pixbuf.from_file (this.get_image_path ("element-enumeration-16.png"));
      this.icon_const = new Gdk.Pixbuf.from_file (this.get_image_path ("element-literal-16.png"));
      this.icon_namespace = new Gdk.Pixbuf.from_file (this.get_image_path ("element-namespace-16.png"));
      this.parser = new Parser ();
      this._sb = new SourceBuffer ("default", null);
    }
    catch (Error e)
    {
      debug ("an error occourred: %s", e.message);
    }
  }

  public ValaProvider (Gtk.TextView text_view)
  {
    this.text_view = text_view;
  }

  public string get_name ()
  {
    return "CompletionValaProvider";
  }

  public List? get_proposals (SourceCompletion.Trigger trigger)
  {
    Timer timer = new Timer ();

    this.transform_result (this.find_proposals ());
    timer.stop ();
    debug ("TOTAL TIME ELAPSED: %f", timer.elapsed ());
    if (list.length () == 0 && this.cache_building)
    {
      this.last_trigger = trigger;
    }
    else
    {
      this.last_trigger = null;
    }
    return #list;
  }

  public void finish ()
  {
    this.list = null;
  }
}

